package com.netease.nis.captcha_plugin_flutter

import android.app.Activity
import android.content.Context
import android.text.TextUtils
import android.util.Log
import com.netease.nis.captcha.Captcha
import com.netease.nis.captcha.CaptchaConfiguration
import com.netease.nis.captcha.CaptchaConfiguration.LangType
import com.netease.nis.captcha.CaptchaListener
import io.flutter.plugin.common.EventChannel
import io.flutter.plugin.common.MethodCall
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.MainScope
import kotlinx.coroutines.launch

/**
 * Created by hzhuqi on 2020/9/21
 */
class CaptchaHelper : CoroutineScope by MainScope() {
    var events: EventChannel.EventSink? = null

    fun init(activity: Activity, call: MethodCall) {
        val captchaId = call.argument<String>("captcha_id")
        val isDebug = call.argument<Boolean>("is_debug") ?: false //是否开启debug模式
        val isNoSenseMode = call.argument<Boolean>("is_no_sense_mode") ?: false//是否智能无感知
        val dimAmount = call.argument<Double>("dimAmount") ?: 0.5 //弹窗透明度
        val controlBarStartUrl = call.argument<String>("control_bar_start_url") ?: "" //自定义滑块开始背景
        val controlBarMovingUrl = call.argument<String>("control_bar_moving_url") ?: ""//自定义滑块滑动背景
        val controlBarErrorUrl = call.argument<String>("control_bar_error_url") ?: "" //自定义滑块错误背景
        val isTouchOutsideDisappear =
            call.argument<Boolean>("is_touch_outside_disappear") ?: true //点击弹窗外部是否可以关闭验证码
        val timeout = call.argument<Int>("timeout") ?: 1000 * 10 //超时时间
        val isHideCloseBtn = call.argument<Boolean>("is_hide_close_button") ?: false //是否隐藏关闭按钮
        val useDefaultFallback = call.argument<Boolean>("use_default_fallback") ?: true //是否采用默认降级
        val failedMaxRetryCount = call.argument<Int>("failed_max_retry_count") ?: 3 //失败后尝试最大次数
        val languageType = call.argument<String>("language_type") ?: "" //多语言语言类型
        val theme = call.argument<String>("theme") ?: "light" // 主题
        val loadingText = call.argument<String>("loading_text") ?: ""// 加载文案
        val apiServer = call.argument<String>("api_server") ?: "" // 私有化接口
        val staticServer = call.argument<String>("static_server") ?: "" // 私有化资源

        if (captchaId != null) {
            val builder = CaptchaConfiguration.Builder()
            builder.captchaId(captchaId)
            builder.listener(captchaListener)
            builder.debug(isDebug)
            if (isNoSenseMode) {
                builder.mode(CaptchaConfiguration.ModeType.MODE_INTELLIGENT_NO_SENSE)
            }
            builder.timeout(timeout.toLong())
            builder.backgroundDimAmount(dimAmount.toFloat())
            builder.failedMaxRetryCount(failedMaxRetryCount)
            if (!TextUtils.isEmpty(controlBarMovingUrl) &&
                !TextUtils.isEmpty(controlBarStartUrl) &&
                !TextUtils.isEmpty(
                    controlBarErrorUrl
                )
            ) {
                builder.controlBarImageUrl(
                    controlBarStartUrl,
                    controlBarMovingUrl,
                    controlBarErrorUrl
                )
            }
            builder.touchOutsideDisappear(isTouchOutsideDisappear)
            builder.useDefaultFallback(useDefaultFallback)
            builder.hideCloseButton(isHideCloseBtn)
            if (!TextUtils.isEmpty(languageType)) {
                builder.languageType(string2LangType(languageType))
            }
            builder.theme(
                if ("light" == theme)
                    CaptchaConfiguration.Theme.LIGHT
                else
                    CaptchaConfiguration.Theme.DARK
            )
            if (!TextUtils.isEmpty(loadingText)) {
                builder.loadingText(loadingText)
            }
            if (!TextUtils.isEmpty(apiServer)) {
                builder.apiServer(apiServer)
            }
            if (!TextUtils.isEmpty(staticServer)) {
                builder.staticServer(staticServer)
            }
            Captcha.getInstance().init(builder.build(activity))
            Log.i("CaptchaHelper", "初始化成功")
        } else {
            Log.e("CaptchaHelper", "业务id必须传")
        }
    }

    fun showCaptcha() {
        Captcha.getInstance().validate()
    }

    fun destroy() {
        Captcha.getInstance().destroy()
    }

    private var captchaListener: CaptchaListener = object : CaptchaListener {
        override fun onReady() {
        }

        override fun onValidate(result: String?, validate: String?, msg: String?) {
            val data = HashMap<String, Any?>()
            data["validate"] = validate
            data["result"] = result
            data["msg"] = msg
            launch(Dispatchers.Main) {
                events?.success(data)
            }
        }

        override fun onError(code: Int, msg: String?) {
            val data = HashMap<String, Any?>()
            data["code"] = code
            data["msg"] = msg
            launch(Dispatchers.Main) {
                events?.success(data)
            }
        }

        override fun onClose(closeType: Captcha.CloseType?) {
            val data = HashMap<String, Any?>()
            data["closed"] = true
            launch(Dispatchers.Main) {
                events?.success(data)
            }
        }
    }

    private fun string2LangType(langTypeStr: String): LangType {
        return when (langTypeStr) {
            "am" -> {
                LangType.LANG_AM
            }
            "ar" -> {
                LangType.LANG_AR
            }
            "as" -> {
                LangType.LANG_AS
            }
            "az" -> {
                LangType.LANG_AZ
            }
            "be" -> {
                LangType.LANG_BE
            }
            "bg" -> {
                LangType.LANG_BG
            }
            "bn" -> {
                LangType.LANG_BN
            }
            "bo" -> {
                LangType.LANG_BO
            }
            "bs" -> {
                LangType.LANG_BS
            }
            "ca" -> {
                LangType.LANG_CA
            }
            "cs" -> {
                LangType.LANG_CS
            }
            "da" -> {
                LangType.LANG_DA
            }
            "de" -> {
                LangType.LANG_DE
            }
            "el" -> {
                LangType.LANG_EL
            }
            "en" -> {
                LangType.LANG_EN
            }
            "en-GB" -> {
                LangType.LANG_EN
            }
            "en-US" -> {
                LangType.LANG_EN_US
            }
            "es" -> {
                LangType.LANG_ES
            }
            "es-la" -> {
                LangType.LANG_ES_LA
            }
            "et" -> {
                LangType.LANG_ET
            }
            "eu" -> {
                LangType.LANG_EU
            }
            "fa" -> {
                LangType.LANG_FA
            }
            "fi" -> {
                LangType.LANG_FI
            }
            "fr" -> {
                LangType.LANG_FR
            }
            "gl" -> {
                LangType.LANG_GL
            }
            "gu" -> {
                LangType.LANG_GU
            }
            "hi" -> {
                LangType.LANG_HI
            }
            "hr" -> {
                LangType.LANG_HR
            }
            "hu" -> {
                LangType.LANG_HU
            }
            "id" -> {
                LangType.LANG_ID
            }
            "it" -> {
                LangType.LANG_IT
            }
            "he" -> {
                LangType.LANG_HE
            }
            "ja" -> {
                LangType.LANG_JA
            }
            "jv" -> {
                LangType.LANG_JV
            }
            "ka" -> {
                LangType.LANG_KA
            }
            "kk" -> {
                LangType.LANG_KK
            }
            "km" -> {
                LangType.LANG_KM
            }
            "kn" -> {
                LangType.LANG_KN
            }
            "ko" -> {
                LangType.LANG_KO
            }
            "lo" -> {
                LangType.LANG_LO
            }
            "lt" -> {
                LangType.LANG_LT
            }
            "lv" -> {
                LangType.LANG_LV
            }
            "mai" -> {
                LangType.LANG_MAI
            }
            "mi" -> {
                LangType.LANG_MI
            }
            "mk" -> {
                LangType.LANG_MK
            }
            "ml" -> {
                LangType.LANG_ML
            }
            "mn" -> {
                LangType.LANG_MN
            }
            "mr" -> {
                LangType.LANG_MR
            }
            "ms" -> {
                LangType.LANG_MS
            }
            "my" -> {
                LangType.LANG_MY
            }
            "no" -> {
                LangType.LANG_NO
            }
            "ne" -> {
                LangType.LANG_NE
            }
            "nl" -> {
                LangType.LANG_NL
            }
            "or" -> {
                LangType.LANG_OR
            }
            "pa" -> {
                LangType.LANG_PA
            }
            "pl" -> {
                LangType.LANG_PL
            }
            "pt" -> {
                LangType.LANG_PT
            }
            "pt-br" -> {
                LangType.LANG_PT_BR
            }
            "ro" -> {
                LangType.LANG_RO
            }
            "ru" -> {
                LangType.LANG_RU
            }
            "si" -> {
                LangType.LANG_SI
            }
            "sk" -> {
                LangType.LANG_SK
            }
            "sl" -> {
                LangType.LANG_SL
            }
            "sr" -> {
                LangType.LANG_SR
            }
            "sv" -> {
                LangType.LANG_SV
            }
            "sw" -> {
                LangType.LANG_SW
            }
            "ta" -> {
                LangType.LANG_TA
            }
            "te" -> {
                LangType.LANG_TE
            }
            "th" -> {
                LangType.LANG_TH
            }
            "fil" -> {
                LangType.LANG_FIL
            }
            "tr" -> {
                LangType.LANG_TR
            }
            "ug" -> {
                LangType.LANG_UG
            }
            "uk" -> {
                LangType.LANG_UK
            }
            "ur" -> {
                LangType.LANG_UR
            }
            "uz" -> {
                LangType.LANG_UZ
            }
            "vi" -> {
                LangType.LANG_VI
            }
            "zh-CN" -> {
                LangType.LANG_ZH_CN
            }
            "zh-HK" -> {
                LangType.LANG_ZH_HK
            }
            "zh-TW" -> {
                LangType.LANG_ZH_TW
            }
            else -> LangType.LANG_ZH_CN
        }
    }
}